@@ -1,17 +1,20 @@
 from distutils.version import LooseVersion
+from functools import partial
 import logging
 from logging.handlers import TimedRotatingFileHandler
 import os
 from pathlib import Path
 import sys
 import warnings
-import uuid
 
 import appdirs
+import msgpack
+import msgpack_numpy as mpn
 
 from IPython import get_ipython
 
 from bluesky_kafka import Publisher
+from event_model import RunRouter
 
 from ._version import get_versions
 
@@ -40,7 +43,8 @@ def configure_base(
     configure_logging=True,
     pbar=True,
     ipython_logging=True,
-    publish_documents_to_kafka=False
+    publish_documents_to_kafka=False,
+    tb_minimize=True,
 ):
     """
     Perform base setup and instantiation of important objects.
@@ -92,7 +96,9 @@ def configure_base(
         True by default. Console output and exception stack traces will be
         written to IPython log file when IPython logging is enabled.
     publish_documents_to_kafka: boolean, optional
-        False by default. If True publish bluesky documents to a Kafka message broker.
+        False by default. If True publish bluesky documents to a Kafka message broker
+    tb_minimize : boolean, optional
+        If IPython should print out 'minimal' tracebacks.
 
     Returns
     -------
@@ -105,6 +111,9 @@ def configure_base(
 
     >>>> configure_base(get_ipython().user_ns, 'chx');
     """
+
+    ipython = get_ipython()
+
     ns = {}  # We will update user_ns with this at the end.
     # Protect against double-subscription.
     SENTINEL = "__nslsii_configure_base_has_been_run"
@@ -166,7 +175,8 @@ def configure_base(
         # Register bluesky IPython magics.
         from bluesky.magics import BlueskyMagics
 
-        get_ipython().register_magics(BlueskyMagics)
+        if ipython:
+            ipython.register_magics(BlueskyMagics)
 
     if bec:
         # Set up the BestEffortCallback.
@@ -197,15 +207,13 @@ def configure_base(
         setup_ophyd()
 
     if configure_logging:
-        configure_bluesky_logging(ipython=get_ipython())
+        configure_bluesky_logging(ipython=ipython)
 
-    if ipython_logging:
+    if ipython_logging and ipython:
         from nslsii.common.ipynb.logutils import log_exception
 
         # IPython logging will be enabled with logstart(...)
-        configure_ipython_logging(
-            exception_logger=log_exception, ipython=get_ipython()
-        )
+        configure_ipython_logging(exception_logger=log_exception, ipython=ipython)
 
     if publish_documents_to_kafka:
         subscribe_kafka_publisher(
@@ -213,14 +221,17 @@ def configure_base(
             beamline_name=broker_name,
             bootstrap_servers="cmb01:9092,cmb02:9092,cmb03:9092",
             producer_config={
-                "enable.idempotence": True,
-                "linger.ms": 0
-            }
+                "acks": 0,
+                "message.timeout.ms": 3000,
+                "queue.buffering.max.kbytes": 10 * 1048576,  # default is 1048576
+                "compression.codec": "snappy"
+            },
         )
 
-    # always configure %xmode minimal
-    # so short tracebacks are printed to the console
-    get_ipython().magic("xmode minimal")
+    if tb_minimize and ipython:
+        # configure %xmode minimal
+        # so short tracebacks are printed to the console
+        ipython.magic("xmode minimal")
 
     # convenience imports
     # some of the * imports are for 'back-compatibility' of a sort -- we have
@@ -266,10 +277,16 @@ def configure_base(
     return list(ns)
 
 
-def configure_bluesky_logging(ipython, appdirs_appname="bluesky"):
+def configure_bluesky_logging(ipython, appdirs_appname="bluesky", propagate_log_messages=False):
     """
     Configure a TimedRotatingFileHandler log handler and attach it to
-    bluesky, ophyd, caproto, and nslsii loggers.
+    bluesky, ophyd, caproto, and nslsii loggers. In addition, by default set
+    the ``propagate`` field on each logger to ``False`` so log messages will
+    not propagate to higher level loggers such as a root logger configured
+    by a user. If you want log messages from these loggers to propagate to
+    higher level loggers simply set ``propagate_log_messages=True`` when
+    calling this function, or set the ``propagate`` field to ``True`` in
+    client code.
 
     The log file path is taken from environment variable BLUESKY_LOG_FILE, if
     that variable has been set. If not the default log file location is determined
@@ -284,6 +301,11 @@ def configure_bluesky_logging(ipython, appdirs_appname="bluesky"):
         appname passed to appdirs.user_log_dir() when the BLUESKY_LOG_FILE
         environment variable has not been set; use the default for production,
         set to something else for testing
+    propagate_log_messages: bool
+        the ``propagate`` field on the bluesky, caproto, nslsii, ophyd, and ipython
+        loggers will be set to this value; if False (the default) log messages
+        from these loggers will not propagate to higher-level loggers
+        (such as a root logger)
 
     Returns
     -------
@@ -301,9 +323,7 @@ def configure_bluesky_logging(ipython, appdirs_appname="bluesky"):
             file=sys.stderr,
         )
     else:
-        bluesky_log_dir = Path(
-            appdirs.user_log_dir(appname=appdirs_appname)
-        )
+        bluesky_log_dir = Path(appdirs.user_log_dir(appname=appdirs_appname))
         if not bluesky_log_dir.exists():
             bluesky_log_dir.mkdir(parents=True, exist_ok=True)
         bluesky_log_file_path = bluesky_log_dir / Path("bluesky.log")
@@ -322,18 +342,17 @@ def configure_bluesky_logging(ipython, appdirs_appname="bluesky"):
         "  %(module)s:%(lineno)d] %(message)s"
     )
     log_file_handler.setFormatter(logging.Formatter(fmt=log_file_format))
-    logging.getLogger("bluesky").addHandler(log_file_handler)
-    logging.getLogger("caproto").addHandler(log_file_handler)
-    logging.getLogger("ophyd").addHandler(log_file_handler)
-    logging.getLogger("nslsii").addHandler(log_file_handler)
-    ipython.log.addHandler(log_file_handler)
-    # set the loggers to send INFO and higher log
-    # messages to their handlers
-    logging.getLogger("bluesky").setLevel("INFO")
-    logging.getLogger("caproto").setLevel("INFO")
-    logging.getLogger("ophyd").setLevel("INFO")
-    logging.getLogger("nslsii").setLevel("INFO")
-    ipython.log.setLevel("INFO")
+
+    for logger_name in ("bluesky", "caproto", "ophyd", "nslsii"):
+        logger = logging.getLogger(logger_name)
+        logger.addHandler(log_file_handler)
+        logger.setLevel("INFO")
+        logger.propagate = propagate_log_messages
+
+    if ipython:
+        ipython.log.addHandler(log_file_handler)
+        ipython.log.setLevel("INFO")
+        ipython.log.propagate = propagate_log_messages
 
     return bluesky_log_file_path
 
@@ -381,12 +400,12 @@ def configure_ipython_logging(
             file=sys.stderr,
         )
     else:
-        bluesky_ipython_log_dir = Path(
-            appdirs.user_log_dir(appname=appdirs_appname)
-        )
+        bluesky_ipython_log_dir = Path(appdirs.user_log_dir(appname=appdirs_appname))
         if not bluesky_ipython_log_dir.exists():
             bluesky_ipython_log_dir.mkdir(parents=True, exist_ok=True)
-        bluesky_ipython_log_file_path = bluesky_ipython_log_dir / Path("bluesky_ipython.log")
+        bluesky_ipython_log_file_path = bluesky_ipython_log_dir / Path(
+            "bluesky_ipython.log"
+        )
         print(
             "environment variable BLUESKY_IPYTHON_LOG_FILE is not set,"
             f" using default file path '{bluesky_ipython_log_file_path}'",
@@ -530,13 +549,14 @@ def migrate_metadata():
 
 def subscribe_kafka_publisher(RE, beamline_name, bootstrap_servers, producer_config):
     """
-    Create and subscribe a Kafka Publisher to a RunEngine. Keep a reference to the Publisher.
-    The Publisher will publish to Kafka topic "<beamline>.bluesky.documents".
+    Subscribe a RunRouter to the specified RE to create Kafka Publishers.
+    Each Publisher will publish documents from a single run to the
+    Kafka topic "<beamline_name>.bluesky.documents".
 
     Parameters
     ----------
     RE: RunEngine
-        the RunEngine to which the Kafka Publisher will be subscribed
+        the RunEngine to which the RunRouter will be subscribed
 
     beamline_name: str
         beamline name, for example "csx", to be used in building the
@@ -550,18 +570,73 @@ def subscribe_kafka_publisher(RE, beamline_name, bootstrap_servers, producer_con
 
     Returns
     -------
-    subscription_token: int
-        use this to unsubscribe the Publisher from the RE this way: ``RE.unsubscribe(subscription_token)``
+    topic: str
+        the Kafka topic on which bluesky documents will be published
+
+    runrouter_token: int
+        subscription token corresponding to the RunRouter subscribed to the RunEngine
+        by this function
 
     """
     topic = f"{beamline_name.lower()}.bluesky.documents"
-    _kafka_publisher = Publisher(
-         topic=topic,
-         bootstrap_servers=bootstrap_servers,
-         key=uuid.uuid4(),
-         producer_config=producer_config
-    )
-    subscription_token = RE.subscribe(_kafka_publisher)
-    logging.getLogger("nslsii").info('RE will publish documents to Kafka topic %s', topic)
 
-    return subscription_token
+    def kafka_publisher_factory(name, start_doc):
+        # create a Kafka Publisher for a single run
+        kafka_publisher = Publisher(
+            topic=topic,
+            bootstrap_servers=bootstrap_servers,
+            key=start_doc["uid"],
+            producer_config=producer_config,
+            flush_on_stop_doc=True,
+            serializer=partial(msgpack.dumps, default=mpn.encode),
+        )
+
+        def handle_publisher_exceptions(name_, doc_):
+            """
+            Do not let exceptions from the Kafka producer kill the RunEngine.
+            This is for testing and is not sufficient for Kafka in production.
+            TODO: improve exception handling for production
+            """
+            try:
+                kafka_publisher(name_, doc_)
+            except Exception:
+                logger = logging.getLogger("nslsii")
+                logger.exception(
+                    "an error occurred when %s published %s\nname: %s\ndoc %s",
+                    kafka_publisher,
+                    name_,
+                    doc_,
+                )
+
+        try:
+            # call Producer.list_topics to test if we can connect to a Kafka broker
+            # TODO: add list_topics method to KafkaPublisher
+            cluster_metadata = kafka_publisher._producer.list_topics(
+                topic=topic, timeout=5.0
+            )
+            logging.getLogger("nslsii").info(
+                "connected to Kafka broker(s): %s", cluster_metadata
+            )
+            return [handle_publisher_exceptions], []
+        # TODO: raise BlueskyException or similar from KafkaPublisher.list_topics
+        except Exception:
+            # For now failure to connect to a Kafka broker will not be considered a
+            # significant problem because we are not relying on Kafka. When and if
+            # we do rely on Kafka for storing documents we will need a more robust
+            # response here.
+            # TODO: improve exception handling for production
+            nslsii_logger = logging.getLogger("nslsii")
+            nslsii_logger.exception("%s failed to connect to Kafka", kafka_publisher)
+
+            # documents will not be published to Kafka brokers
+            return [], []
+
+    rr = RunRouter(factories=[kafka_publisher_factory])
+    runrouter_token = RE.subscribe(rr)
+
+    # log this only once
+    logging.getLogger("nslsii").info(
+        "RE will publish documents to Kafka topic %s", topic
+    )
+
+    return topic, runrouter_token
